import xapi from 'xapi';

const serverUrl = 'https://touch10.1table.cn:8188'

let platform = '';      //系统类型
let uri = '';           //终端号码
let s_number = '';      //序列号
let currentVerNum = ''; //系统版本
let phonenumber = ''    //注册手机号

init();

//***初始化
async function init(){
  uri = await xapi.Config.sip.uri.get();
  platform = await xapi.Status.SystemUnit.ProductPlatform.get();
  s_number = await xapi.Status.SystemUnit.Hardware.Module.SerialNumber.get();
  currentVerNum = await xapi.Status.Provisioning.Software.Current.VersionId.get().then((ret) =>{
    return ret.split(' ')[0].replace('ce', '').replace('tc', '')
  });

  xapi.config.set('HttpClient Mode:', 'On').then(()=>{console.log('HttpClient Mode设置为On')}).catch((error) => {console.error(error);});
}

xapi.event.on('UserInterface Extensions Widget Action', async (event) => {
   //平板控制软件激活
  if (event.WidgetId === 'button—active' && event.Type === 'clicked') {
    //检查设备是否已注册
    if (uri === '' || uri === null) {
      console.log("SIP URI get failed");
      return Yaqingda_showalert('请确认该设备是否已注册,如已注册请忽略！', 10);
    }

    let isactive = await checkActived(uri);
    if (!isactive) {
      showTextInput('请输入手机号', '获取验证码', 'phonenumber' );
    }else{
      //安装宏软件
      console.log('在线升级开始...')
      macroUprade();
    }
  }  else if (event.WidgetId === 'button-uninstall' && event.Type === 'pressed') {
    //平板控制软件卸载
    showPromptOptions('确认卸载？','unitall','卸载','取消')
  } 

})

xapi.event.on('UserInterface Message TextInput Response', (event) => {
  if (event.FeedbackId === 'phonenumber') {
    phonenumber = event.Text
    if (isPoneAvailable(phonenumber)) {
      sendsms(phonenumber)
    }else{
      showTextInput('请输入正确的手机号', '获取验证码', 'phonenumber' );
    }
    
  }else if (event.FeedbackId === 'smscode') {
    var smsCode = event.Text
    if (smsCode.length !== 6) {
      showPromptOptions("请输入正确的验证码", 'smscode', 'OK')
      return
    }
    active(phonenumber, event.Text)
  }
})

xapi.event.on('UserInterface Message Prompt Response', async (event) => {
  if (event.FeedbackId === 'phonenumber') {
    showTextInput('请输入手机号', '获取验证码', 'phonenumber' );
  } else if (event.FeedbackId === 'smscode') {
    xapi.command('UserInterface Message TextInput Display', {
      FeedbackId: 'smscode',
      Title: '请输入短信验证码',
      Text: '请输入',
    });
  } else if ((event.FeedbackId === 'upgrade_system') && event.OptionId === '1') {
    systemUprade();
  } else if (event.FeedbackId === 'unitall' &&  event.OptionId === '1') {
    console.log('卸载宏程序开始');
    xapi.command(' Macros Macro RemoveAll').then((result) => {
      xapi.command(' UserInterface Extensions Clear').then((result) => {
        showPromptMsg('平板控制软件已卸载...');
        setTimeout(async function () {
          xapi.command('UserInterface Extensions Panel Close');
        }, 2000);
        console.log('卸载宏程序完成');

      }).catch(handleError);
    }).catch(handleError);
  } 
});

//发送验证码
async function sendsms(phonenumber) {
  var payload = JSON.stringify({
    'phonenumber': phonenumber,
    'command': 'requestdySmsCode',
    'user_name': uri,
    's_number': s_number
  });

  console.log(payload);

  xapi.command('HttpClient Post', {
    Header: ["Content-Type: application/json"],
    Url: serverUrl,
    ResultBody: "PlainText"
  },
    payload)
    .then((result) => {
      var body = JSON.stringify(result.Body)
      console.log("body:" + body)

      var jsonBody = JSON.parse(result.Body)
      if (jsonBody.status) {
        showPromptOptions(jsonBody.msg, 'smscode', 'OK')
      } else {
        showPromptOptions(jsonBody.msg, 'phonenumber', 'OK')
      }
    })
    .catch((err) => {
      console.log("failed: " + err.message)
      Yaqingda_showalert("系统错误,请联系客服");
    });
}

//激活
async function active(phonenumber, smscode) {
  var payload = JSON.stringify({
    'phonenumber': phonenumber,
    'command': 'active',
    'sms_key': smscode,
    'user_name': uri,
    's_number': s_number,
    // 'key': key
  });

  console.log(payload);

  xapi.command('HttpClient Post', {
    Header: ["Content-Type: application/json"],
    Url: serverUrl,
    ResultBody: "PlainText"
  },payload) .then((result) => {
      var strR = JSON.stringify(result)
      console.log("验证码校验:" + strR)

      var jsonBody = JSON.parse(result.Body)
      if (jsonBody.status === 0) {
        showPromptMsg("激活成功!");
        macroUprade();
      } else {
        showPromptOptions(jsonBody.msg, 'smscode', 'OK')
        return;
      }
    })
    .catch((err) => {
      console.log("failed: " + err.message);
      showPromptMsg("激活失败，请联系客服!");
    });

}

/**
 * 检查平板控制软件是否需要更新
 */
async function macroUprade() {
  //9.8以下不支持Get,默认升级到9.8
  if (compareVersion('9.8.0', currentVerNum)) {
    let version = await xapi.Status.Provisioning.Software.Current.VersionId.get()
    return showPromptOptions(`系统版本为${version}，请升级系统版本后再更新宏软件！`, 'upgrade_system', '升级系统软件', '暂不升级');
  }

  console.log('宏程序更新开始')

  var payload = JSON.stringify({
    'platform':platform,
    'currentVerNum':currentVerNum,
    'uri': uri.split('@')[0],
    's_number': s_number
  });

  console.log(payload);
  
  xapi.command('HttpClient Post', {
    Header: ["Content-Type: application/json"],
    Url: serverUrl+"/macroDownloadInfo"
    ,ResultBody: "PlainText"
  },
    payload)
    .then((result) => {
      console.log(`macroDownloadInfo:${result.Body}`)
      var body = JSON.parse(result.Body)
      var code = body.code;
      if(code == 0){
      var macrodownload_url = body.data[0].download_url;
      console.log('macrodownload_url:',macrodownload_url)
      xapi.command('Provisioning Service Fetch', {
        url: macrodownload_url,
        Mode: 'Replace'
      }).then(() => {
        console.log("Provisioning Service Fetch successfully")
      })

      var counter = 0
      var count = 600
      var timer = setInterval(function (event) {
        xapi.command('UserInterface Message Prompt Display', {
          Title: '提示',
          Text: "请稍候，宏软件更新中(" + counter + "s)"
        });
        counter++;
        if (counter >= count) {
          clearInterval(timer)
        }
      }, 1000)
    } else {
      Yaqingda_showalert("系统错误！")
    }
  })
  .catch((err) => {
    console.log("checkUprade failed: " + err.message)
    Yaqingda_showalert("系统错误！")
  });
}

/**
 * 更新系统软件
 */
async function systemUprade() {
  const products = [
    { platform: ["DX80", "DX70"], producttype: 1, download_url: 'https://www.1table.cn/OSS/client/dxseries/s52040ce9_8_1-0f71ac008c3.pkg' },
    { platform: ["Codec", "Codec Plus", "Room Kit", "Room 55", "Room 55D", "Room 70", "Room 70D","Room 70S", "Board 55", "Board 55S", "Board 70", "Board 70S", "Board 85S", "Room Kit Mini"], producttype: 2, download_url: 'https://www.1table.cn/OSS/client/roomkit/cmterm-s53200ce9_8_1-0f71ac008c3.k3.cop.sgn' },
    { platform: ["SX10"], producttype: 3, download_url: 'https://www.1table.cn/OSS/client/sxseries/SX10/s52030ce9_8_1-0f71ac008c3.pkg' },
    { platform: ["CodecPro", "Codec Pro", "Room70 G2","Room 70D G2","Room 70S G2",  "Desk","Desk Pro","Desk Mini","Board Pro 55","Board Pro 75"], producttype: 4, download_url: 'https://www.1table.cn/OSS/client/roomkitpro/cmterm-s53300ce9_8_1-0f71ac008c3.k3.cop.sgn' },
    { platform: ["SX20", "MX200 G2","MX300 G2"], producttype: 5, download_url: 'https://www.1table.cn/OSS/client/sxseries/SX20/s52010ce9_8_1-0f71ac008c3.pkg' },
    { platform: ["SX80", "MX700", "MX700ST", "MX700D","MX800","MX800S","MX800ST","MX800D"], producttype: 6, download_url: 'https://www.1table.cn/OSS/client/sxseries/SX80 MX700 800/s52020ce9_8_1-0f71ac008c3.pkg' },
    { platform: ["Room Kit EQ","Room Bar","Room Bar Pro"], producttype: 7, download_url: 'https://www.1table.cn/OSS/client/roomkitpro/cmterm-s53300ce9_8_1-0f71ac008c3.k3.cop.sgn' },
    { platform: ["Room 70 G2","Room 70 Panorama","Room Panorama"], producttype: 8, download_url: 'https://www.1table.cn/OSS/client/roomkitpro/cmterm-s53300ce9_8_1-0f71ac008c3.k3.cop.sgn' },
    { platform: ["Room USB"], producttype: 9, download_url: 'https://www.1table.cn/OSS/client/roomkit/cmterm-s53200ce11_5_2_4.k4.cop.sha512' },
  ];

let systemUrl = ''
  for (const product of products) {
    if (product.platform.includes(platform)) {
      systemUrl = product.download_url;
      break;
    }
  }
  if(systemUrl){
    showPromptOptions('系统软件升级中...');
    xapi.command('SystemUnit SoftwareUpgrade', {
      url: systemUrl,
      Forced: false
    }).then((result) => {
       showPromptOptions('系统软件升级中...');
      console.log("SoftwareUpgrade", result);
    }).catch(handleError);
  }else{
    Yaqingda_showalert("该设备暂不支持系统升级，请联系客服！")
  }
}

async function checkActived (user_name) {

  return await xapi.command('HttpClient Get', {
    Header: ["Content-Type: application/json"],
    Url: serverUrl + "/isactivedAccount?user_name=" + user_name 
  }).then((result) => {
      console.log(`注册检查:${result.Body}`)
      let body = JSON.parse(result.Body)
      let isactive = body.status
      return isactive;
    })
    .catch((err) => {
      console.log("panel active failed: " + err.message)
    });
}


function handleError (error) {
  console.log('Error', error);
  showPromptMsg('系统异常:' + error.message);
}

/**
 * 手机号码校验
 * @param {*} str 
 */
function isPoneAvailable (str) {
  var myreg = /^[1][3,4,5,6,7,8,9][0-9]{9}$/;
  if (!myreg.test(str)) {
    Yaqingda_showalert('请填写正确的手机号码')
    return false;
  } else {
    return true;
  }
}

function compareVersion (aVersion, bVersion) {
  var a = (aVersion != null ? aVersion : "");
  var b = (bVersion != null ? bVersion : "");
  var aStringArray = a.split(".");
  var bStringArray = b.split(".");
  var loopCount = Math.min(aStringArray.length, bStringArray.length);
  for (var i = 0; i < loopCount; i++) {
    if (parseInt(aStringArray[i]) > parseInt(bStringArray[i])) {
      return true;
    }
    else if (parseInt(aStringArray[i]) < parseInt(bStringArray[i])) {
      return false;
    }
  }
  return aStringArray.length > bStringArray.length;
}

function showTextInput (title, text, feedbackId) {
  xapi.command('UserInterface Message TextInput Display', {
    FeedbackId: feedbackId,
    Title: title,
    Text: text,
  });
}

function Yaqingda_showalert (text, duration = 5) {
  xapi.command('UserInterface Message Alert Display', {
    Title: '提示',
    Text: text,
    Duration: duration,
  });
}

function showPromptMsg (text) {
  xapi.command('UserInterface Message Prompt Display', {
    Title: '提示',
    Text: text,
    Duration: 5
  });
}

function showPromptOptions (text, feedbackId, ...options) {
  let value = {
    Title: '提示',
    Text: text
  }
  if(feedbackId){
    value.FeedbackId = feedbackId;
  }
  for(let i = 0; i < options.length; i++){
    value['Option.'+(i+1)] = options[i];
  } 
  xapi.command('UserInterface Message Prompt Display', value);
}